% GABFRMATXXL.M   Gabor Frame Matrix 
% by XXL, 13.10.2004
%
% based on GABFRMATRIX.M   Sigang Qiu  11.11.93,  generated by DIASUNS
%
% This algorithms creates the Gabor frame matrix with the window g,
% the time shift a and the frequency shift b. It is a very fast and 
% efficient way of calculating the Gabor frame matrix.
%  
% inputs : g .... the Gabor window (a vector of length n)
%          a .... time shift parameter (lattice parameter)
%          b .... frequency shift parameter (lattice parameter)
%
%  output: the n x n Gabor frame matrix .
%
%  usage:  S = Gabfrmatxxl(g,a,b)
%
%  see also: Blo2WalXXL.m, GabFrMatrix.m

% Example:
%         n = 144;
%         a = 9;
%         b = 12;
%         g = gaussnk(n);
%         S = gabfrmatxxl(g, a, b);
%
% Changes:
% this function now only calculates the non-zero-block-matrix, which then is
% used by Blo2WalXXL to create S.
%
% (Gabfrmatrix.m): This is the same version as gabgab with the block and 
% rotrotm replaced  by the trick codes.
%
% XXL .. Peter Balazs, contact: a8927259@unet.univie.ac.at

function UU = gabfrmatXXL(g, a, b);
  
n = length(g);

% error checking not necessary is done in the next function
u = blockxxl(g,a,b);

UU = Blo2WalXXL(u,n);

% ----------------------------------
% profile on;
% profile report;
% figure(2);
% profile plot;
% profile off;

% OLD:
% T = n/a; F = n/b;
%if (rem(n, a) ~=0) |(rem(n, b) ~=0); 
%disp('Check the size of the matrix, please'); return; end
%gg = [g, g];
% uu = zeros(a,b);
% for jj=1:a,
%   for rr = 1:b;
%      per1 = jj:a:(n+jj-1);
%      pp = jj + (rr - 1)*F;
%      per2 = pp:a:(n+pp-1);
%      uu(jj,rr) = gg(per1)*gg(per2)'; 
%    end
% end
