%  builds the inverse of the blockmatrix A by using of preconditioners
%  Inputs:  S .... b x a - Blockmatrix
%           n .... size of original full-matrix
%  Output:  T .... result
%           dat... differences between the partial sums (if not -> 0 =>
%           error)
% 
%  Author: Peter Balazs, 15.2.2005
%  extension of blockinv.m by Mario Hampejs, 3.6.2004

n=144;
g=gaussnk(n);
a=12;b=9;
S= blockxxl(g,a,b); 

% [b,a]=size(S);

O=zeros(b,a);
I=O;
I(1,:)=ones(1,a);

If=eye(n);

egs = sort(real(eig(blo2walxxl(S,n))));    % eigenvalues sorted descending
B = max(egs), A = min(egs),
lopt = 2/(A+B);   



disp('1: No preconditioning ')
A=S*lopt;            % Original Matrix
disp('Abstand von Identity: l2-Norm, Walnut-norm, Janssen-l1-norm, HS-Norm,')
[norm(bm2fm(A,n)-If), walnorm(A-I),jansnormblo(A-I,a),sqrt(n/a)*norm(A-I,'fro')]
[S1,dat1]=blockeulersum(A,n); % inverse of A
disp('Konvergenzgeschwindigkeit:')
dat1


disp('2: Multiplication - left side ')
P1=O;
P1(1,:)=oneover(S(1,:));
A=blockm(S,P1,n);            % Preconditionierte Matrix
disp('Abstand von Identity: l2-Norm, Walnut-norm, Janssen-l1-norm, HS-Norm,')
[norm(bm2fm(A,n)-If), walnorm(A-I),jansnormblo(A-I,a),sqrt(n/a)*norm(A-I,'fro')]
[A2,dat2]=blockeulersum(A,n); % inverse of A
S2= blockm(P1,A2,n);         % inverse of S
disp('Konvergenzgeschwindigkeit:')
dat2

disp('3: convolution operator left side')
if a ~= 1 
    % important distinction, as the 'mean' program handles a [1 x b] matrix
    % differently than a [a x b] matrix.
    v = mean(S.'); 
else
    v = S.';
end;
w=ifft(oneover(fft(v))); % deconvolution
P2=w.'*ones(1,a);
A=blockm(S,P2,n);            % Preconditionierte Matrix
disp('Abstand von Identity: l2-Norm, Walnut-norm, Janssen-l1-norm, HS-Norm,')
[norm(bm2fm(A,n)-If), walnorm(A-I),jansnormblo(A-I,a),sqrt(n/a)*norm(A-I,'fro')]
[A3,dat3]=blockeulersum(A,n); % inverse of A
S3= blockm(P2,A3,n);         % inverse of S
disp('Konvergenzgeschwindigkeit:')
dat3

disp('4: first multiplication then convolution (from left)')
P1=O;
P1(1,:)=oneover(S(1,:));
A=blockm(S,P1,n);  
if a ~= 1 
    v = mean(A.'); 
else
    v = A.';
end;
w=ifft(oneover(fft(v))); % deconvolution
P2=w.'*ones(1,a);
A=blockm(A,P2,n); 
P3=blockm(P1,P2,n);
disp('Abstand von Identity: l2-Norm, Walnut-norm, Janssen-l1-norm, HS-Norm,')
[norm(bm2fm(A,n)-If), walnorm(A-I),jansnormblo(A-I,a),sqrt(n/a)*norm(A-I,'fro')]
[A4,dat4]=blockeulersum(A,n);
S4= blockm(P3,A4,n);         % inverse of S
disp('Konvergenzgeschwindigkeit:')
dat4 


figure(2);
p1 = semilogy(dat1,'r'); hold on;
p2 = semilogy(dat2,'g'); hold on;
p3 = semilogy(dat3,'b'); hold on;
p4 = semilogy(dat4,'m'); 
legend('frame algorithm', 'diagonal preconditioning', 'circulant preconditioning','double preconditioning'); 


