function   CD=col2diagxxl(M); 
% Reorder matrix to switch the columns to the sidediagonals
% col2diagxxl.M,   20.04.2005  XXL 
% 
% This program reorders the matrix, switching the entries from the same 
% column to the same side-diagonal. The first row stays fixed.
% CD_i,j = M_{i,i-j}
%
% Inputs :  M ... square n x n matrix
%
% Output :  CD ... n x n matrix
%
% usage :    DM  =  col2diagxxl(M); 
%  
% see also: diag2row, SIDEDIGM, Blo2WalXXl, Wal2BloXXL
% 
% Author: XXL ..  a8927259@unet.univie.ac.at 
%              or  xxl@kfs.oeaw.ac.at
%
% Notes: this is exactly the operation used for the Kohn-Nirenberg symbol!
%
% Lit.: [XXL] Peter Balazs, "Irregular and regular Gabor multiplier with
%             application to psychoacoustic masking"
% 
% Copyright :  (c) NUHAG, Faculty of Math., University of Vienna, AUSTRIA
%              http://nuhag.mat.uunivie.ac.at/
%              (c) Acoustics Research Institute, Austrian Academy of
%              Science
%              http://www.kfs.oeaw.ac.at
%
%              Permission is granted to modify and re-distribute this
%              code in any manner as long as this notice is preserved.
%              All standard disclaimers apply.


if nargin < 1;  error('no input'); end;  
[h,n] = size(M);  
if h ~= n ; 
    disp('M has to be square matrix')
    return; 
end;

CD = zeros(n,n);  

jj = 1 : n; 
for ii = 1 : n;
CD(ii,jj)=M(ii,mod(jj-ii,n)+1);
end;   
