function   DM=diag2col(M,F); 
% Make the diagonals of a matrix the columns of a new matrix
% diag2col.M,   12.01.2005  XXL 
% 
% This program produces a compressed collection of the side diagonals
% as columns of a new matrix, the program transfers the  
% sidediagonals to the columns of the new matrix, only using every
% F-th diagonal. 
% This program uses the main diagonal as first column and
% the F-th sidediagonal right of the main diagonal as second column
% and so on: 
% DM_i,j = M_{i,i+j*F}
%
% Inputs :  M ... square n x n matrix
%           F ... a divisor of n
%                 (Default) F = 1; 
%
% Output :  DM ... n/F x n   matrix, 
%
% usage :    DM  =  diag2col(M,F); 
%  
% see also: diag2row, SIDEDIGM, Blo2WalXXl, Wal2BloXXL
% 
% Author: XXL ..  a8927259@unet.univie.ac.at 
%              or  xxl@kfs.oeaw.ac.at
% based on:
%       SIDEDIGM.M,   18.11.93 by  Sigang Qiu 
% 
% Copyright :  (c) NUHAG, Faculty of Math., University of Vienna, AUSTRIA
%              http://nuhag.mat.uunivie.ac.at/
%              (c) Acoustics Research Institute, Austrian Academy of
%              Science
%              http://www.kfs.oeaw.ac.at
%
%              Permission is granted to modify and re-distribute this
%              code in any manner as long as this notice is preserved.
%              All standard disclaimers apply.


if nargin == 1;  F = 1; end;  
[h,n] = size(M);  
if h ~= n ; 
    disp('M has to be square matrix')
    return; 
end;
if mod(n,F) ~= 0;
    sprintf('F (= %g) should be a divisor of n (%g)',F,n)
    return;
end;

DM = zeros(n,n/F);  

for jj = 1 : n/F; 
tt = mod((jj-1)*F:(jj-1)*F+n,h)+1;
temp = M(:,tt);
DM(:,jj)=diag(temp);
end;   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
