function   DM=diag2row(M,F); 
% Make the diagonals of a matrix the rows of a new matrix
% diag2col.M,   29.10.04  XXL 
% 
% This program produces a compressed collection of the side diagonals
% as rows of a new matrix, the program transfers the  
% sidediagonals to the rows of the new matrix, only using every
% F-th diagonal. 
% This program uses the main diagonal as first row and
% the F-th sidediagonal under the main diagonal as second row
% and so on: 
% DM_i,j = M_{i*F+j,j}
%
% Inputs :  M ... square n x n matrix
%           F ... a divisor of n
%                 (Default) F = 1; 
%
% Output :  DM ... n/F x n   matrix, 
%
% usage :    DM  =  diag2row(M,F); 
%  
% see also: diag2col, SIDEDIGM, Blo2WalXXl, Wal2BloXXL
% 
% Author: XXL ..  a8927259@unet.univie.ac.at 
%              or  xxl@kfs.oeaw.ac.at
% based on:
%       SIDEDIGM.M,   18.11.93 by  Sigang Qiu 
% 
% Copyright :  (c) NUHAG, Faculty of Math., University of Vienna, AUSTRIA
%              http://nuhag.mat.uunivie.ac.at/
%              (c) Acoustics Research Institute, Austrian Academy of
%              Science
%              http://www.kfs.oeaw.ac.at
%
%              Permission is granted to modify and re-distribute this
%              code in any manner as long as this notice is preserved.
%              All standard disclaimers apply.

if nargin == 1;  F = 1; end;  
[h,n] = size(M);  
if h ~= n ; 
    disp('M has to be square matrix')
    return; 
end;
if mod(n,F) ~= 0;
    sprintf('F (= %g) should be a divisor of n (%g)',F,n)
    return;
end;

DM = zeros(n/F,n);  

for jj = 1 : n/F; 
    tt = mod((jj-1)*F:(jj-1)*F+n,h)+1;
    temp = M(tt,:);
    DM(jj,:)=diag(temp);
end;   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Old code:
% st = (jj-1)*F*n ;   
% ndx = rem(st-1 + (1: (n+1) : n^2), n^2) + 1 ;   
% sd = M(ndx); 
% DM(jj,:) =  sd;
% 
% st = (jj-1)*F*n ;   
% ndx = rem(st-1 + (1: (n+1) : n^2), n^2) + 1 ;   
% sd = M(ndx); 
% DM(jj,:) =  sd;
% translation matrix(j):
% P = rot(eye(n),k);
