function   KN=kohnniren(M); 
% Calculate the Kohn-Nirenberg symbol of M
% kohniren.M,   20.04.2005  XXL 
% 
% This program calculates the Kohn-Nirenberg Symbol of the matrix M. In the
% finite-dimensional case the matrix corresponds to the integration kernel
% and therefore:
% KN (M) = F_2 T_a M = fft(col2diag(M)); (see [XXL])
%
% Inputs :  M ... square n x n matrix
%
% Output :  CD ... n x n matrix: the Kohn-Nirenberg symbol
%
% usage :    DM  =  kohnniren(M); 
%  
% see also: ker2kohn.m (same functionality, no comments)
% 
% Author: XXL ..  a8927259@unet.univie.ac.at 
%              or  xxl@kfs.oeaw.ac.at
%
% Notes: this is exactly the operation used for the Kohn-Nirenberg symbol!
%
% Lit.: [XXL] Peter Balazs, "Irregular and regular Gabor multiplier with
%             application to psychoacoustic masking"
% 
% Copyright :  (c) NUHAG, Faculty of Math., University of Vienna, AUSTRIA
%              http://nuhag.mat.uunivie.ac.at/
%              (c) Acoustics Research Institute, Austrian Academy of
%              Science
%              http://www.kfs.oeaw.ac.at
%
%              Permission is granted to modify and re-distribute this
%              code in any manner as long as this notice is preserved.
%              All standard disclaimers apply.


if nargin < 1;  error('no input'); end;  
[h,n] = size(M);  
if h ~= n ; 
    disp('M has to be square matrix')
    return; 
end;

KN = fft(col2diagxxl(M).');
